pub trait AsMatMut: AsMatRef {
// Required method
fn as_mat_mut(&mut self) -> MatMut<'_, Self::T, Self::Rows, Self::Cols>;
}Expand description
trait for types that can be converted to a matrix view
pub trait AsMatMut: AsMatRef {
// Required method
fn as_mat_mut(&mut self) -> MatMut<'_, Self::T, Self::Rows, Self::Cols>;
}trait for types that can be converted to a matrix view